/*
**  CXSC is a C++ library for eXtended Scientific Computing (V 2.2)
**
**  Copyright (C) 1990-2000 Institut fuer Angewandte Mathematik,
**                          Universitaet Karlsruhe, Germany
**            (C) 2000-2006 Wiss. Rechnen/Softwaretechnologie
**                          Universitaet Wuppertal, Germany   
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Library General Public
**  License as published by the Free Software Foundation; either
**  version 2 of the License, or (at your option) any later version.
**
**  This library is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
**  Library General Public License for more details.
**
**  You should have received a copy of the GNU Library General Public
**  License along with this library; if not, write to the Free
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
**
**  F. Blomquist, University of Wuppertal, 20.09.2007;
*/

/* CVS $Id: lx_complex.inl,v 1.3 2006/12/06 18:10:25 cxsc Exp $ */

namespace cxsc {

inline lx_real Re(const lx_complex &a)
{ return a.re; }

inline lx_real Im(const lx_complex &a)
{ return a.im; }

inline int StagPrec(const lx_complex &a) throw() 
{ return StagPrec(a.re); }

inline int expoRe(const lx_complex &a) throw()
{ return expo(a.re); }

inline int expoIm(const lx_complex &a) throw()
{ return expo(a.im); }

inline l_real lr_partRe(const lx_complex &a) throw()
{ return lr_part(a.re); }

inline l_real lr_partIm(const lx_complex &a) throw()
{ return lr_part(a.im); }

inline lx_complex & SetRe(lx_complex &a, const lx_real &b)
{ a.re = b; return a; } // The real part of a is substituted by b.
inline lx_complex & SetRe(lx_complex &a, const l_real &b)
{ a.re = b; return a; } // The real part of a is substituted by b.
inline lx_complex & SetRe(lx_complex &a, const real &b)
{ a.re = b; return a; } // The real part of a is substituted by b.

inline lx_complex & SetIm(lx_complex &a, const lx_real &b)
{ a.im = b; return a; } // The imaginary part of a is substituted by b.
inline lx_complex & SetIm(lx_complex &a, const l_real &b)
{ a.im = b; return a; } // The imaginary part of a is substituted by b.
inline lx_complex & SetIm(lx_complex &a, const real &b)
{ a.im = b; return a; } // The imaginary part of a is substituted by b.

inline bool operator == (const lx_complex &a, const lx_complex &b) 
    throw() { return (a.re == b.re && a.im == b.im); }

inline bool operator == (const lx_complex &a, const l_complex &b) 
    throw() { return (a.re == Re(b) && a.im == Im(b)); }
inline bool operator == (const lx_complex &a, const complex &b) 
    throw() { return (a.re == Re(b) && a.im == Im(b)); }
inline bool operator == (const l_complex &a, const lx_complex &b) 
    throw() { return b == a; }
inline bool operator == (const complex &a, const lx_complex &b) 
    throw() { return (b == a); }

inline bool operator == (const lx_complex &a, const lx_real &b) throw()
{ return a.re == b && a.im == 0.0; }
inline bool operator == (const lx_complex &a, const l_real &b) throw()
{ return a.re == b && a.im == 0.0; }
inline bool operator == (const lx_complex &a, const real &b) throw()
{ return a.re == b && a.im == 0.0; }

inline bool operator == (const lx_real &a, const lx_complex &b) throw()
{ return a == b.re && b.im == 0.0; }
inline bool operator == (const l_real &a, const lx_complex &b) throw()
{ return a == b.re && b.im == 0.0; }
inline bool operator == (const real &a,   const lx_complex &b) throw()
{ return a == b.re && b.im == 0.0; }

inline bool operator != (const lx_complex &a, const lx_complex &b) throw()
{ return !(a == b); }

inline bool operator != (const lx_complex &a, const l_complex &b) throw()
{ return !(a == b); }
inline bool operator != (const lx_complex &a, const complex   &b) throw()
{ return !(a == b); }
inline bool operator != (const l_complex &a, const lx_complex &b) throw()
{ return !(a == b); }
inline bool operator != (const complex   &a, const lx_complex &b) throw()
{ return !(a == b); }

inline bool operator != (const lx_complex &a, const lx_real &b) throw()
{ return !(a == b); }
inline bool operator != (const lx_complex &a, const l_real &b) throw()
{ return !(a == b); }
inline bool operator != (const lx_complex &a, const real &b)   throw()
{ return !(a == b); }
inline bool operator != (const lx_real &a, const lx_complex &b) throw()
{ return !(a == b); }
inline bool operator != (const l_real &a, const lx_complex &b) throw()
{ return !(a == b); }
inline bool operator != (const real &a,   const lx_complex &b) throw()
{ return !(a == b); }

inline lx_complex operator + (const lx_complex &a) throw()
{ return a; }
inline lx_complex operator - (const lx_complex &a) throw()
{ return lx_complex(-a.re,-a.im); }


// -----------------------------------------------------

inline std::ostream& operator << (std::ostream& s, const lx_complex& a) 
    throw()
// A value a of type lx_complex is written to the output channel.
// The output has the form:  { ? , ? }
{     
    s << '(' 
      << a.re 
      << ", " 
      << a.im       
      << ')';
    return s;
}

inline std::string & operator << (std::string &s, const lx_complex& a) 
    throw()
// The value of a variable a of type lx_complex is copied to a string s.
// s has the form:  { ? , ? }
{  
    std::stringstream ss;
    string str;

    ss.clear();
    str = "({(";
    s += str;
    ss << expo(a.re);
    ss >> str;
    s += str;
    s+="),"; 
    ss.clear();
    ss << lr_part(a.re);
    ss >> str;
    s += str;
    s +="}, {("; 
    ss.clear();
    ss << expo(a.im);
    ss >> str;
    s += str;
    s+="),"; 
    ss.clear();
    ss << lr_part(a.im);
    ss >> str;
    s += str;
    s+="})";
    return s;
}

} // end namespace cxsc
