/*
**  CXSC is a C++ library for eXtended Scientific Computing (V 2.2)
**
**  Copyright (C) 1990-2000 Institut fuer Angewandte Mathematik,
**                          Universitaet Karlsruhe, Germany
**            (C) 2000-2006 Wiss. Rechnen/Softwaretechnologie
**                          Universitaet Wuppertal, Germany   
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Library General Public
**  License as published by the Free Software Foundation; either
**  version 2 of the License, or (at your option) any later version.
**
**  This library is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
**  Library General Public License for more details.
**
**  You should have received a copy of the GNU Library General Public
**  License along with this library; if not, write to the Free
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
**
**  F. Blomquist, University of Wuppertal, 19.09.2007;
*/
/* CVS $Id: lx_interval.inl,v 1.3 2006/12/06 18:10:25 cxsc Exp $ */

namespace cxsc {

inline int StagPrec(const lx_interval &a) throw() 
{ return StagPrec(a.li); }

inline int expo(const lx_interval &a) throw()
{ return (a.ex); }

inline l_interval li_part(const lx_interval &a) throw()
{ return (a.li); }

inline int Disjoint(const lx_interval& a, const lx_interval& b)
{ return (Inf(a)>Sup(b) || Inf(b)>Sup(a)); }

inline int in(const lx_interval& x, const lx_interval& y) 
{ return ( (Inf(y) < Inf(x)) && (Sup(x) < Sup(y)) ); }

inline int in(const l_interval& x, const lx_interval& y) 
{ return ( (Inf(y) < Inf(x)) && (Sup(x) < Sup(y)) ); }

inline int in(const interval& x, const lx_interval& y) 
{ return ( (Inf(y) < Inf(x)) && (Sup(x) < Sup(y)) ); }

inline int in(const lx_real& x, const lx_interval& y ) 
{ return ( (Inf(y) <= x) && (x <= Sup(y)) ); }

inline int in(const l_real& x, const lx_interval& y ) 
{ return ( (Inf(y) <= x) && (x <= Sup(y)) ); }

inline int in(const real& x, const lx_interval& y ) 
{ return ( (Inf(y) <= x) && (x <= Sup(y)) ); }

inline lx_interval Blow( const lx_interval& x, const real& eps )
{
    l_interval lx(li_part(x));
    return lx_interval( expo(x), Blow(lx,eps) );
}

inline lx_real AbsMin (const lx_interval &x)
{
   if ( in(0.0,x) ) return lx_real(0.0);
   else 
   { 
       lx_real y(Inf(x));
       if (y > 0.0) return y;
       else return -Sup(x);
   }
}
  
inline lx_real AbsMax (const lx_interval &x)
{ 
    lx_real a, b;

    a = abs(Inf(x));
    b = abs(Sup(x));

    if (a > b)
	return a;
    else
	return b;
}

inline lx_interval & lx_interval::operator = (const lx_interval &a) throw()
{
    ex = a.ex;
    li = a.li;
    return *this;
} 

inline lx_interval & lx_interval::operator = (const lx_real &a) throw()
{
    ex = expo(a);
    li = lr_part(a);
    return *this;
} 


inline lx_interval & lx_interval::operator = (const l_interval &a) throw()
{
    ex = 0;
    li = a;
    return *this;
}

inline lx_interval & lx_interval::operator = (const l_real & a) throw()
{
    ex = 0;
    li = a;
    return *this;
}

inline lx_interval & lx_interval::operator = (const real & a) throw()
{
    ex = 0;
    li = a;
    return *this;
}

inline lx_interval & lx_interval::operator = (const interval & a) throw()
{
    ex = 0;
    li = a;
    return *this;
} 


inline lx_interval operator+(const lx_interval & a) throw() { return a; }
inline lx_interval operator-(const lx_interval & a) throw()
{  return lx_interval(a.ex,-a.li);  }

inline lx_interval operator + (const lx_interval &a, const l_interval &b) throw()
{  return a + lx_interval(b); }

inline lx_interval operator + (const l_interval &a, const lx_interval &b) throw()
{  return lx_interval(a) + b; }

inline lx_interval operator + (const lx_interval &a, const l_real &b) throw()
{  return a + lx_interval(b); }

inline lx_interval operator + (const l_real &a, const lx_interval &b) throw()
{  return lx_interval(a) + b; }

inline lx_interval operator + (const lx_interval &a, const lx_real &b) throw()
{  return a + lx_interval(b); }

inline lx_interval operator + (const lx_real &a, const lx_interval &b) throw()
{  return lx_interval(a) + b; }

inline lx_interval operator + (const lx_interval &a, const real &b) throw()
{  return a + lx_interval(b); }

inline lx_interval operator + (const real &a, const lx_interval &b) throw()
{  return lx_interval(a) + b; }

inline lx_interval operator + (const lx_interval &a, const interval &b) throw()
{  return a + lx_interval(b); }

inline lx_interval operator + (const interval &a, const lx_interval &b) throw()
{  return lx_interval(a) + b; }

inline lx_interval & operator +=(lx_interval &a, const lx_interval &b) throw()
{  return a = a+b; }

inline lx_interval & operator +=(lx_interval &a, const l_interval &b) throw()
{  return a = a+b; }

inline lx_interval & operator +=(lx_interval &a, const l_real &b) throw()
{  return a = a+b; }

inline lx_interval & operator +=(lx_interval &a, const lx_real &b) throw()
{  return a = a+b; }

inline lx_interval & operator +=(lx_interval &a, const real &b) throw()
{  return a = a+b; }

inline lx_interval & operator +=(lx_interval &a, const interval &b) throw()
{  return a = a+b; }

inline lx_interval operator - (const lx_interval &a, const lx_interval &b) throw()
{  return a + lx_interval(-b); }

inline lx_interval operator - (const lx_interval &a, const l_interval &b) throw()
{  return a + lx_interval(-b); }

inline lx_interval operator - (const l_interval &a, const lx_interval &b) throw()
{  return lx_interval(a) + (-b); }
 
inline lx_interval operator - (const lx_interval &a, const l_real &b) throw()
{  return a + lx_interval(-b); }

inline lx_interval operator - (const l_real &a, const lx_interval &b) throw()
{  return lx_interval(a) + (-b); }

inline lx_interval operator - (const lx_interval &a, const lx_real &b) throw()
{  return a + lx_interval(-b); }

inline lx_interval operator - (const lx_real &a, const lx_interval &b) throw()
{  return lx_interval(a) + (-b); }
 
inline lx_interval operator - (const lx_interval &a, const real &b) throw()
{  return a + lx_interval(-b); }

inline lx_interval operator - (const real &a, const lx_interval &b) throw()
{  return lx_interval(a) + (-b); }

inline lx_interval operator - (const lx_interval &a, const interval &b) throw()
{  return a + lx_interval(-b); }

inline lx_interval operator - (const interval &a, const lx_interval &b) throw()
{  return lx_interval(a) + (-b); }

inline lx_interval & operator -=(lx_interval &a, const lx_interval &b) throw()
{  return a = a-b; }
inline lx_interval & operator -=(lx_interval &a, const l_interval &b) throw()
{  return a = a-b; }
inline lx_interval & operator -=(lx_interval &a, const l_real &b) throw()
{  return a = a-b; }
inline lx_interval & operator -=(lx_interval &a, const lx_real &b) throw()
{  return a = a-b; }
inline lx_interval & operator -=(lx_interval &a, const real &b) throw()
{  return a = a-b; }
inline lx_interval & operator -=(lx_interval &a, const interval &b) throw()
{  return a = a-b; }


inline lx_interval operator * (const lx_interval &a, const l_interval &b) throw()
{
    return a * lx_interval(b);
}

inline lx_interval operator * (const l_interval &a, const lx_interval &b) throw()
{
    return lx_interval(a) * b;
}

inline lx_interval operator * (const lx_interval &a, const l_real &b) throw()
{
    return a * lx_interval(b);
}

inline lx_interval operator * (const l_real &a, const lx_interval &b) throw()
{
    return lx_interval(a) * b;
}

inline lx_interval operator * (const lx_interval &a, const lx_real &b) throw()
{
    return a * lx_interval(b);
}

inline lx_interval operator * (const lx_real &a, const lx_interval &b) throw()
{
    return lx_interval(a) * b;
}

inline lx_interval operator * (const lx_interval &a, const real &b) throw()
{
    return a * lx_interval(b);
}

inline lx_interval operator * (const real &a, const lx_interval &b) throw()
{
    return lx_interval(a) * b;
}

inline lx_interval operator * (const lx_interval &a, const interval &b) throw()
{
    return a * lx_interval(b);
}

inline lx_interval operator * (const interval &a, const lx_interval &b) throw()
{
    return lx_interval(a) * b;
}

inline lx_interval & operator *=(lx_interval &a, const lx_interval &b) throw()
{  return a = a*b; }
inline lx_interval & operator *=(lx_interval &a, const l_interval &b) throw()
{  return a = a*b; }
inline lx_interval & operator *=(lx_interval &a, const l_real &b) throw()
{  return a = a*b; }
inline lx_interval & operator *=(lx_interval &a, const lx_real &b) throw()
{  return a = a*b; }
inline lx_interval & operator *=(lx_interval &a, const real &b) throw()
{  return a = a*b; }
inline lx_interval & operator *=(lx_interval &a, const interval &b) throw()
{  return a = a*b; } 

inline lx_interval operator / (const lx_interval &a, const l_interval &b) 
                              throw(ERROR_LINTERVAL_DIV_BY_ZERO)
{ return a / lx_interval(b); }

inline lx_interval operator / (const l_interval &a, const lx_interval &b) 
                              throw(ERROR_LINTERVAL_DIV_BY_ZERO)
{ return lx_interval(a) / b; }

inline lx_interval operator / (const lx_interval &a, const l_real &b) 
                              throw(ERROR_LINTERVAL_DIV_BY_ZERO)
{ return a / lx_interval(b); }

inline lx_interval operator / (const l_real &a, const lx_interval &b) 
                              throw(ERROR_LINTERVAL_DIV_BY_ZERO)
{ return lx_interval(a) / b; }

inline lx_interval operator / (const lx_interval &a, const real &b) 
                              throw(ERROR_LINTERVAL_DIV_BY_ZERO)
{ return a / lx_interval(b); }

inline lx_interval operator / (const real &a, const lx_interval &b) 
                              throw(ERROR_LINTERVAL_DIV_BY_ZERO)
{ return lx_interval(a) / b; }

inline lx_interval operator / (const lx_interval &a, const interval &b) 
                              throw(ERROR_LINTERVAL_DIV_BY_ZERO)
{ return a / lx_interval(b); }

inline lx_interval operator / (const interval &a, const lx_interval &b) 
                              throw(ERROR_LINTERVAL_DIV_BY_ZERO)
{ return lx_interval(a) / b; }

inline lx_interval operator / (const lx_interval &a, const lx_real &b) 
                                      throw(ERROR_LINTERVAL_DIV_BY_ZERO)
{  return a / lx_interval(b); }

inline lx_interval operator / (const lx_real &a, const lx_interval &b) 
                                      throw(ERROR_LINTERVAL_DIV_BY_ZERO)
{  return lx_interval(a) / b; }


inline lx_interval & operator /=(lx_interval &a, const lx_interval &b) throw()
{  return a = a/b; }
inline lx_interval & operator /=(lx_interval &a, const l_interval &b) throw()
{  return a = a/b; }
inline lx_interval & operator /=(lx_interval &a, const l_real &b) throw()
{  return a = a/b; }
inline lx_interval & operator /=(lx_interval &a, const real &b) throw()
{  return a = a/b; }
inline lx_interval & operator /=(lx_interval &a, const interval &b) throw()
{  return a = a/b; }
inline lx_interval & operator /=(lx_interval &a, const lx_real &b) throw()
{  return a = a/b; } 

// ------------------ Vergleichsoperatoren ----------------------------------

inline bool operator ! (const lx_interval& a) throw()
{ return !a.li; }

inline bool operator == (const lx_interval &a, const lx_interval &b) throw()
{
    l_interval al(li_part(a)), bl(li_part(b));
    int na(expo(a)), nb(expo(b)), exa(expo_gr(al)), exb(expo_gr(bl)),d;
    bool a_0(exa<-100000), b_0(exb<-100000), res;

    if (a_0 || b_0) res = (a_0 == b_0);
    else // a,b <> 0:
    {
	d = exa-exb;
	if (d>0)
	{  // bl nach oben skalieren:
	    Times2pown(bl,d);
	    nb = sub_int(nb,d);  // b = 2^nb * bl;
	} else // d<=0:
	{  // al nach oben skalieren:
	    Times2pown(al,-d);
	    na = add_int(na,d);
	}
	res = (na==nb && al==bl);
    }
    return res;
}

inline bool operator == (const lx_interval &a, const l_interval &b) throw()
{  return (a == lx_interval(b)); }

inline bool operator == (const l_interval &a, const lx_interval &b) throw()
{  return ( lx_interval(a) == b); }

inline bool operator == (const lx_interval &a, const interval &b) throw()
{  return (a == lx_interval(b)); }

inline bool operator == (const interval &a, const lx_interval &b) throw()
{  return ( lx_interval(a) == b); }

inline bool operator == (const lx_interval &a, const real &b) throw()
{  return ( a == lx_interval(b)); }

inline bool operator == (const lx_interval &a, const l_real &b) throw()
{  return ( a == lx_interval(b)); }

inline bool operator == (const real &a, const lx_interval &b) throw()
{  return ( lx_interval(a) == b); }

inline bool operator == (const l_real &a, const lx_interval &b) throw()
{  return ( lx_interval(a) == b); }

inline bool operator == (const lx_interval &a, const lx_real &b) throw()
{  return ( a == lx_interval(b)); }

inline bool operator == (const lx_real &a, const lx_interval &b) throw()
{  return ( lx_interval(a) == b); }


inline bool operator != (const lx_interval &a, const lx_interval &b) throw()
{ return !(a==b); }

inline bool operator != (const lx_interval &a, const l_interval &b) throw()
{ return !(a==lx_interval(b)); }

inline bool operator != (const l_interval &a, const lx_interval &b) throw()
{ return !(lx_interval(a) == b); }

inline bool operator != (const lx_interval &a, const interval &b) throw()
{ return !(a==lx_interval(b)); }

inline bool operator != (const interval &a, const lx_interval &b) throw()
{ return !(lx_interval(a) == b); }

inline bool operator != (const lx_interval &a, const real &b) throw()
{ return !(a == lx_interval(b)); }

inline bool operator != (const real &a, const lx_interval &b) throw()
{ return !(lx_interval(a) == b); }

inline bool operator != (const lx_interval &a, const l_real &b) throw()
{ return !(a == lx_interval(b)); }

inline bool operator != (const l_real &a, const lx_interval &b) throw()
{ return !(lx_interval(a) == b); }

inline bool operator != (const lx_interval &a, const lx_real &b) throw()
{ return !(a == lx_interval(b)); }

inline bool operator != (const lx_real &a, const lx_interval &b) throw()
{ return !(lx_interval(a) == b); }

// ------------------------------------------------------------------------
// --------------------------- set comparisons ----------------------------
// ------------------------------------------------------------------------

// ---- lx_interval--lx_interval
inline bool operator <  (const lx_interval &a, const lx_interval &b) throw()
{ return( Inf(a)>Inf(b) && Sup(a)<Sup(b) ); }
inline bool operator <= (const lx_interval &a, const lx_interval &b) throw()
{ return( Inf(a)>=Inf(b) && Sup(a)<=Sup(b) ); }
inline bool operator > (const lx_interval& a, const lx_interval& b) throw()
{ return(Inf(a)<Inf(b) && Sup(a)>Sup(b)); }
inline bool operator >= (const lx_interval& a, const lx_interval& b) throw()
{ return(Inf(a)<=Inf(b) && Sup(a)>=Sup(b)); }

// ---- lx_interval--l_interval
inline bool operator <  (const lx_interval &a, const l_interval &b) throw()
{ return( Inf(a)>Inf(b) && Sup(a)<Sup(b) ); }
inline bool operator <= (const lx_interval &a, const l_interval &b) throw()
{ return( Inf(a)>=Inf(b) && Sup(a)<=Sup(b) ); }
inline bool operator <  (const l_interval &a, const lx_interval &b) throw()
{ return( Inf(a)>Inf(b) && Sup(a)<Sup(b) ); }
inline bool operator <= (const l_interval &a, const lx_interval &b) throw()
{ return( Inf(a)>=Inf(b) && Sup(a)<=Sup(b) ); }
inline bool operator > (const lx_interval& a, const l_interval& b) throw()
{ return(Inf(a)<Inf(b) && Sup(a)>Sup(b)); }
inline bool operator >= (const lx_interval& a, const l_interval& b) throw()
{ return(Inf(a)<=Inf(b) && Sup(a)>=Sup(b)); }
inline bool operator > (const l_interval& a, const lx_interval& b) throw()
{ return(Inf(a)<Inf(b) && Sup(a)>Sup(b)); }
inline bool operator >= (const l_interval& a, const lx_interval& b) throw()
{ return(Inf(a)<=Inf(b) && Sup(a)>=Sup(b)); }

// ---- lx_interval--interval
inline bool operator <  (const lx_interval &a, const interval &b) throw()
{ return( Inf(a)>Inf(b) && Sup(a)<Sup(b) ); }
inline bool operator <= (const lx_interval &a, const interval &b) throw()
{ return( Inf(a)>=Inf(b) && Sup(a)<=Sup(b) ); }
inline bool operator <  (const interval &a, const lx_interval &b) throw()
{ return( Inf(a)>Inf(b) && Sup(a)<Sup(b) ); }
inline bool operator <= (const interval &a, const lx_interval &b) throw()
{ return( Inf(a)>=Inf(b) && Sup(a)<=Sup(b) ); }
inline bool operator > (const lx_interval& a, const interval& b) throw()
{ return(Inf(a)<Inf(b) && Sup(a)>Sup(b)); }
inline bool operator >= (const lx_interval& a, const interval& b) throw()
{ return(Inf(a)<=Inf(b) && Sup(a)>=Sup(b)); }
inline bool operator > (const interval& a, const lx_interval& b) throw()
{ return(Inf(a)<Inf(b) && Sup(a)>Sup(b)); }
inline bool operator >= (const interval& a, const lx_interval& b) throw()
{ return(Inf(a)<=Inf(b) && Sup(a)>=Sup(b)); }

// ---- lx_interval--real
inline bool operator <  (const real &a, const lx_interval &b) throw()
{ return( a>Inf(b) && a<Sup(b) ); }
inline bool operator <= (const real &a, const lx_interval &b) throw()
{ return( a>=Inf(b) && a<=Sup(b) ); }
inline bool operator >  (const lx_interval &a, const real &b) throw()
{ return( b>Inf(a) && b<Sup(a) ); }
inline bool operator >=  (const lx_interval &a, const real &b) throw()
{ return( b>=Inf(a) && b<=Sup(a) ); }

// ---- lx_interval--l_real
inline bool operator <  (const l_real &a, const lx_interval &b) throw()
{ return( a>Inf(b) && a<Sup(b) ); }
inline bool operator <= (const l_real &a, const lx_interval &b) throw()
{ return( a>=Inf(b) && a<=Sup(b) ); }
inline bool operator >  (const lx_interval &a, const l_real &b) throw()
{ return( b>Inf(a) && b<Sup(a) ); }
inline bool operator >=  (const lx_interval &a, const l_real &b) throw()
{ return( b>=Inf(a) && b<=Sup(a) ); }

// ---- lx_interval--lx_real
inline bool operator <  (const lx_real &a, const lx_interval &b) throw()
{ return( a>Inf(b) && a<Sup(b) ); }
inline bool operator <= (const lx_real &a, const lx_interval &b) throw()
{ return( a>=Inf(b) && a<=Sup(b) ); }
inline bool operator >  (const lx_interval &a, const lx_real &b) throw()
{ return( b>Inf(a) && b<Sup(a) ); }
inline bool operator >=  (const lx_interval &a, const lx_real &b) throw()
{ return( b>=Inf(a) && b<=Sup(a) ); }


inline lx_interval adjust(const lx_interval &a) throw()
{  return lx_interval(a.ex,adjust(a.li));  }

inline lx_interval abs(const lx_interval &a) throw()
{  return lx_interval(a.ex,abs(a.li));  }

inline bool point_intv(const lx_interval &a)
{  return point_intv(a.li);  }

inline void times2pown(lx_interval &a, int n) throw()
{  a = lx_interval(add_int(n,a.ex),a.li); }

inline void times2pown_neg(lx_interval &a, int n)
// Calculating an inclusion of  a*2^n  , with n = 0,-1,-2,...,-2147483647;
// n MUST not be positive!
// Zero MUST not be in the interior of an interval a.
// The above conditions are not tested in this function!
// Allowed intervals, t>0: 
//        [0,0], [0,t], [t,t], [t/2,t], [-t,0], [-t,-t], [-t,-t/2];
// Not allowed intervals:  [-t,t], [-2t,t].
// If a=[0,t], or a=[t,t], or a=[t/2,t] and if t*2^n > 0 is sufficiently 
// small (expo(a)+n<-2147483647) then the calculated inclusion of a*2^n 
// is an interval [0,S], with a*2^n <= S.  
// Blomquist, 16.11.2007;  
{  
    int exal(expo_gr(a.li));
    double db_a,db_al;
    bool neg;

    if (exal>-100000) // a != [0,0]
    {
	db_a = a.ex; db_al = exal;
	if (db_a+n < -2147483647)
	{
	    neg = Inf(a.li)<0;
	    db_a = db_a + db_al + n;
	    if (db_a < -2147484670.0)
		a = lx_interval(lx_real(0),lx_real(-2147483647,MinReal));
	    else 
	    {
		db_a = db_a + 2147483648.0;
		exal = (int) db_a;
		a = lx_interval(lx_real(0),lx_real(-2147483647,comp(0.5,exal+1)));
	    }
	    if (neg) a = -a;
	}
	else // n+a.ex >= -2147483647, so integer overflow not possible!
	    a = lx_interval(n+a.ex,a.li); 
    }
} // times2pown_neg(...)

inline l_real RelDiam( const lx_interval &a )
{ return RelDiam(a.li); }

inline lx_real Inf(const lx_interval &a) throw()
{  return lx_real(a.ex,Inf(a.li));  }

inline lx_real Sup(const lx_interval &a) throw()
{  return lx_real(a.ex,Sup(a.li));  }

inline lx_real diam(const lx_interval &a) throw()
{  return lx_real(a.ex,diam(a.li)); }

inline lx_real mid(const lx_interval& a) throw()
{  return lx_real(a.ex,mid(a.li)); }

inline bool IsEmpty(const lx_interval& a) throw()
{ return  Inf(a.li) > Sup(a.li); }

// ----------------------------- Convex hull -------------------------------

inline lx_interval operator |(const lx_interval &a, const lx_interval &b) throw() 
{
   return lx_interval( (Inf(a)<Inf(b)) ? Inf(a) : Inf(b),
                      (Sup(a)>Sup(b)) ? Sup(a) : Sup(b) );
}

inline lx_interval operator |(const lx_interval &a, const l_interval &b) throw() 
{
    lx_interval Lb(0,b);
    return lx_interval( (Inf(a)<Inf(Lb)) ? Inf(a) : Inf(Lb),
		       (Sup(a)>Sup(Lb)) ? Sup(a) : Sup(Lb) );
}

inline lx_interval operator |(const l_interval &a, const lx_interval &b) throw() 
{
    lx_interval La(0,a);
    return lx_interval( (Inf(La)<Inf(b)) ? Inf(La) : Inf(b),
		       (Sup(La)>Sup(b)) ? Sup(La) : Sup(b) );
}

inline lx_interval operator |(const lx_interval &a, const interval &b) throw() 
{
    lx_interval Lb(0,l_interval(b));
    return lx_interval( (Inf(a)<Inf(Lb)) ? Inf(a) : Inf(Lb),
		       (Sup(a)>Sup(Lb)) ? Sup(a) : Sup(Lb) );
}

inline lx_interval operator |(const interval &a, const lx_interval &b) throw() 
{
    lx_interval La(0,l_interval(a));
    return lx_interval( (Inf(La)<Inf(b)) ? Inf(La) : Inf(b),
		       (Sup(La)>Sup(b)) ? Sup(La) : Sup(b) );
}

inline lx_interval & operator |= (lx_interval &a,const lx_interval &b) throw() 
{
   Inf(a)=(Inf(a)<Inf(b))?Inf(a):Inf(b),Sup(a)=(Sup(a)>Sup(b))?Sup(a):Sup(b);
   return a;
}

inline lx_interval & operator |= (lx_interval &a,const l_interval &b) throw() 
{
    lx_interval Lb(0,b);
    Inf(a) = (Inf(a)<Inf(Lb)) ? Inf(a) : Inf(Lb),
	Sup(a) = (Sup(a)>Sup(Lb)) ? Sup(a) : Sup(Lb);
    return a;
}

inline lx_interval & operator |= (lx_interval &a,const interval &b) throw() 
{
    lx_interval Lb(0,l_interval(b));
    Inf(a) = (Inf(a)<Inf(Lb)) ? Inf(a) : Inf(Lb),
	Sup(a) = (Sup(a)>Sup(Lb)) ? Sup(a) : Sup(Lb);
    return a;
}

inline lx_interval operator | (const lx_real &a, const lx_interval &b) throw() 
{
    return lx_interval( (a<Inf(b)) ? a : Inf(b),
                       (a>Sup(b)) ? a : Sup(b) );
}

inline lx_interval operator | (const real &a, const lx_interval &b) throw() 
{
    lx_real La(a);
    return lx_interval( (La<Inf(b)) ? La : Inf(b),
                       (La>Sup(b)) ? La : Sup(b) );
}

inline lx_interval operator | (const l_real &a, const lx_interval &b) throw() 
{
    lx_real La(0,a);
    return lx_interval( (La<Inf(b)) ? La : Inf(b),
                       (La>Sup(b)) ? La : Sup(b) );
}

inline lx_interval operator | (const lx_interval &a, const lx_real &b) throw() 
{
    return lx_interval( (Inf(a)<b) ? Inf(a) : b,
                       (Sup(a)>b) ? Sup(a) : b );
}

inline lx_interval operator | (const lx_interval &a, const real &b) throw() 
{
    lx_real Lb(b);
    return lx_interval( (Inf(a)<Lb) ? Inf(a) : Lb,
                       (Sup(a)>Lb) ? Sup(a) : Lb );
}

inline lx_interval operator | (const lx_interval &a, const l_real &b) throw() 
{
    lx_real Lb(0,b);
    return lx_interval( (Inf(a)<Lb) ? Inf(a) : Lb,
                       (Sup(a)>Lb) ? Sup(a) : Lb );
}

inline lx_interval & operator |= (lx_interval &a, const real &b) throw() 
{
    lx_real Lb(b);
    Inf(a) = (Inf(a)<Lb) ? Inf(a) : Lb, Sup(a) = (Sup(a)>Lb) ? Sup(a) : Lb;
    return a;
}

inline lx_interval & operator |= (lx_interval &a, const l_real &b) throw() 
{
    lx_real Lb(0,b);
    Inf(a) = (Inf(a)<Lb) ? Inf(a) : Lb, Sup(a) = (Sup(a)>Lb) ? Sup(a) : Lb;
    return a;
}


inline lx_interval & operator |= (lx_interval &a, const lx_real &b) throw() 
{
    Inf(a) = (Inf(a)<b) ? Inf(a) : b, Sup(a) = (Sup(a)>b) ? Sup(a) : b;
    return a;
}

inline lx_interval operator |(const lx_real &a, const lx_real &b) throw()
{
    if(a>b) return lx_interval(b,a);
    else    return lx_interval(a,b);
}

// --------------------------- Intersection -----------------------------

inline lx_interval operator & (const lx_interval &a, const lx_interval &b) 
    throw(ERROR_LINTERVAL_EMPTY_INTERVAL) 
{
    return lx_interval( (Inf(a)>Inf(b)) ? Inf(a) : Inf(b),
                       (Sup(a)<Sup(b)) ? Sup(a) : Sup(b));
}

inline lx_interval operator & (const lx_interval &a, const l_interval &b) 
    throw(ERROR_LINTERVAL_EMPTY_INTERVAL) 
{
    lx_interval Lb(0,b);
    return lx_interval( (Inf(a)>Inf(Lb)) ? Inf(a) : Inf(Lb),
                       (Sup(a)<Sup(Lb)) ? Sup(a) : Sup(Lb));
}

inline lx_interval & operator &= (lx_interval &a, const l_interval &b) 
    throw(ERROR_LINTERVAL_EMPTY_INTERVAL) 
{
    lx_interval Lb(0,b);
    Inf(a) = (Inf(a)>Inf(Lb)) ? Inf(a) : Inf(Lb),
	Sup(a) = (Sup(a)<Sup(Lb)) ? Sup(a) : Sup(Lb);
    if (Inf(a)>Sup(a))
	cxscthrow(ERROR_LINTERVAL_EMPTY_INTERVAL("inline lx_interval & operator &=(lx_interval &a,const l_interval &b)"));
    return a;
}

inline lx_interval operator & (const l_interval &a, const lx_interval &b) 
    throw(ERROR_LINTERVAL_EMPTY_INTERVAL) 
{
    lx_interval La(0,a);
    return lx_interval( (Inf(La)>Inf(b)) ? Inf(La) : Inf(b),
                       (Sup(La)<Sup(b)) ? Sup(La) : Sup(b));
}

inline lx_interval operator & (const lx_interval &a, const interval &b) 
    throw(ERROR_LINTERVAL_EMPTY_INTERVAL) 
{
    lx_interval Lb(0,l_interval(b));
    return lx_interval( (Inf(a)>Inf(Lb)) ? Inf(a) : Inf(Lb),
                       (Sup(a)<Sup(Lb)) ? Sup(a) : Sup(Lb));
}

inline lx_interval & operator &= (lx_interval &a, const interval &b) 
    throw(ERROR_LINTERVAL_EMPTY_INTERVAL) 
{
    lx_interval Lb(0,l_interval(b));
    Inf(a) = (Inf(a)>Inf(Lb)) ? Inf(a) : Inf(Lb),
	Sup(a) = (Sup(a)<Sup(Lb)) ? Sup(a) : Sup(Lb);
    if (Inf(a)>Sup(a))
	cxscthrow(ERROR_LINTERVAL_EMPTY_INTERVAL("inline lx_interval & operator &=(lx_interval &a,const interval &b)"));
    return a;
}

inline lx_interval operator & (const interval &a, const lx_interval &b) 
    throw(ERROR_LINTERVAL_EMPTY_INTERVAL) 
{
    lx_interval La(0,l_interval(a));
    return lx_interval( (Inf(La)>Inf(b)) ? Inf(La) : Inf(b),
                       (Sup(La)<Sup(b)) ? Sup(La) : Sup(b));
}

inline lx_interval & operator &= (lx_interval &a, const lx_interval &b) 
    throw(ERROR_LINTERVAL_EMPTY_INTERVAL) 
{
    Inf(a)=(Inf(a)>Inf(b))?Inf(a):Inf(b),Sup(a)=(Sup(a)<Sup(b))?Sup(a):Sup(b);
    if (Inf(a)>Sup(a))
	cxscthrow(ERROR_LINTERVAL_EMPTY_INTERVAL("inline lx_interval & operator &=(lx_interval &a,const lx_interval &b)"));
   return a;
}

inline lx_interval operator & (const lx_interval &a, const lx_real &b) 
    throw(ERROR_LINTERVAL_EMPTY_INTERVAL) 
{
   return lx_interval( (Inf(a)>b) ? Inf(a) : b,
                      (Sup(a)<b) ? Sup(a) : b );
}

inline lx_interval operator & (const lx_interval &a, const real &b) 
    throw(ERROR_LINTERVAL_EMPTY_INTERVAL) 
{
    lx_real Lb(b);
    return lx_interval( (Inf(a)>Lb) ? Inf(a) : Lb,
		       (Sup(a)<Lb) ? Sup(a) : Lb );
}

inline lx_interval operator & (const lx_interval &a, const l_real &b) 
    throw(ERROR_LINTERVAL_EMPTY_INTERVAL) 
{
    lx_real Lb(0,b);
    return lx_interval( (Inf(a)>Lb) ? Inf(a) : Lb,
		       (Sup(a)<Lb) ? Sup(a) : Lb );
}

inline lx_interval operator & (const lx_real &a, const lx_interval &b) 
    throw(ERROR_LINTERVAL_EMPTY_INTERVAL) 
{
    return lx_interval( (a>Inf(b)) ? a : Inf(b),
                       (a<Sup(b)) ? a : Sup(b) );
}

inline lx_interval operator & (const real &a, const lx_interval &b) 
    throw(ERROR_LINTERVAL_EMPTY_INTERVAL) 
{
    lx_real La(a);
    return lx_interval( (La>Inf(b)) ? La : Inf(b),
                       (La<Sup(b)) ? La : Sup(b) );
}

inline lx_interval operator & (const l_real &a, const lx_interval &b) 
    throw(ERROR_LINTERVAL_EMPTY_INTERVAL) 
{
    lx_real La(0,a);
    return lx_interval( (La>Inf(b)) ? La : Inf(b),
                       (La<Sup(b)) ? La : Sup(b) );
}

inline lx_interval & operator &= (lx_interval &a,const lx_real &b) 
    throw(ERROR_LINTERVAL_EMPTY_INTERVAL) 
{
    Inf(a) = (Inf(a)>b) ? Inf(a) : b, Sup(a) = (Sup(a)<b) ? Sup(a) : b;
    if(Inf(a)>Sup(a))
	cxscthrow(ERROR_LINTERVAL_EMPTY_INTERVAL("inline lx_interval & operator &=(lx_interval &a,const lx_real &b)"));
    return a;
}

inline lx_interval & operator &= (lx_interval &a, const real &b) 
    throw(ERROR_LINTERVAL_EMPTY_INTERVAL) 
{
    lx_real Lb(b);
    Inf(a) = (Inf(a)>Lb) ? Inf(a) : Lb, Sup(a) = (Sup(a)<Lb) ? Sup(a) : Lb;
    if(Inf(a)>Sup(a))
	cxscthrow(ERROR_LINTERVAL_EMPTY_INTERVAL("inline lx_interval & operator &=(lx_interval &a,const real &b)"));
    return a;
}

inline lx_interval & operator &= (lx_interval &a, const l_real &b) 
    throw(ERROR_LINTERVAL_EMPTY_INTERVAL) 
{
    lx_real Lb(0,b);
    Inf(a) = (Inf(a)>Lb) ? Inf(a) : Lb, Sup(a) = (Sup(a)<Lb) ? Sup(a) : Lb;
    if(Inf(a)>Sup(a))
	cxscthrow(ERROR_LINTERVAL_EMPTY_INTERVAL("inline lx_interval & operator &=(lx_interval &a,const l_real &b)"));
    return a;
}

// ------------------------- SetInf, SetSup -----------------------------

inline lx_interval & SetInf(lx_interval& a, const lx_real& b) 
    throw(ERROR_LINTERVAL_EMPTY_INTERVAL)
{
    return a = lx_interval(b,Sup(a));
}

inline lx_interval & SetInf(lx_interval& a, const l_real& b) 
    throw(ERROR_LINTERVAL_EMPTY_INTERVAL)
{
    return a = lx_interval(lx_real(0,b),Sup(a));
}

inline lx_interval & SetInf(lx_interval& a, const real& b) 
    throw(ERROR_LINTERVAL_EMPTY_INTERVAL)
{
    return a = lx_interval(lx_real(0,l_real(b)),Sup(a));
}

inline lx_interval & SetSup(lx_interval& a, const lx_real& b) 
    throw(ERROR_LINTERVAL_EMPTY_INTERVAL)
{
    return a = lx_interval(Inf(a),b);
}

inline lx_interval & SetSup(lx_interval& a, const l_real& b) 
    throw(ERROR_LINTERVAL_EMPTY_INTERVAL)
{
    return a = lx_interval(Inf(a),lx_real(0,b));
}

inline lx_interval & SetSup(lx_interval& a, const real& b) 
    throw(ERROR_LINTERVAL_EMPTY_INTERVAL)
{
    return a = lx_interval(Inf(a),lx_real(0,l_real(b)));
}


} // end namespace cxsc
